package com.aquahopper.suna.lib.net;

/*
FtpClient.java :  an ftp client class.
Copyright (C) 2000, Sunagawa Youichi <suna@love.email.ne.jp>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

import java.net.*;
import java.io.*;

public class FtpClient
{
	private	Socket			cSocket		=	null;
	public	PrintWriter		cOutput		=	null;
	public	BufferedReader	cInput		=	null;

	private	final int 		FTP_POTR	=	21;

	private	String			_sHost		=	"";
	private	String			_sUser		=	"";
	private	String			_sPass		=	"";
	private	String			_sAns		=	"";
public FtpClient()
{
	super();
}
private void _closeSockets()
	throws	UnknownHostException,
			IOException
{
	if(cSocket != null){
		cSocket.close();
	}
}
private Socket _createDataSock(String ftpcmd)
{
	String portcmd = "PORT ";
	int i;
	Socket dataSocket = null;
	try{
		byte[]			addr	= InetAddress.getLocalHost().getAddress();
		ServerSocket	ss 		= new ServerSocket(0,1);
		for(i = 0; i < 4; ++i){
			portcmd = portcmd +  (addr[i] & 0xff) + ",";
		}
		portcmd = portcmd	+ (((ss.getLocalPort()) / 256) & 0xff)
							+ ","
							+ (ss.getLocalPort() & 0xff);
		cOutput.println	(portcmd);
		cOutput.flush	();
		cOutput.println	(ftpcmd);
		cOutput.flush	();
		dataSocket = ss.accept();
		ss.close();
		_sAns = _readCSock();
	}
	catch(Exception e){
		e.printStackTrace();
	}
	return  dataSocket ;
}
private void _createSockets()
	throws	UnknownHostException,
			IOException
{
	cSocket = new Socket		(_sHost, FTP_POTR);
	cOutput = new PrintWriter	(cSocket.getOutputStream());
	cInput  = new BufferedReader(new InputStreamReader(cSocket.getInputStream()));
	_sAns	= _readCSock();
}
private void _doAscii()
{
	_doFtpControl("TYPE A");
}
private void _doBinary()
{
	_doFtpControl("TYPE I");
}
private String _doFtpCommand(String ftpcommand)
{
	String	sRc = "";
	if(ftpcommand == null){
		return("");
	}
	else if(ftpcommand.equals("")){
		return("");
	}
	else{
	}
	
	try{
		int n;
		byte[] buff = new byte[1024];
		Socket				dataSocket	= _createDataSock(ftpcommand);
		BufferedInputStream dataInput	= new BufferedInputStream(dataSocket.getInputStream());
		while((n = dataInput.read(buff)) > 0){
			sRc = sRc + new String(buff,0,n);
		}
		dataSocket.close();
		_readCSock();
	}
	catch(Exception e){
		e.printStackTrace();
	}
	return(sRc);
}
private String _doFtpControl(String ftpcommand)
{
	String	sRc = "";
	try{
		cOutput.println(ftpcommand);
		cOutput.flush();
		sRc = _readCSock();
	}
	catch(Exception e){
		e.printStackTrace();
	}
	return(sRc);
}
private void _doGet(String sRemoteFile, String sLocalFile)
	throws	IOException
{
	int n;
	byte[] buff = new byte[1024];
	FileOutputStream	of	= new FileOutputStream(sLocalFile);
	Socket				ds	= _createDataSock("RETR " + sRemoteFile);
	BufferedInputStream	di	= new BufferedInputStream(ds.getInputStream());
	while((n = di.read(buff)) > 0){
		of.write(buff,0,n);
	}
	ds	.close();
	of	.close();
	_sAns = _readCSock();
}
private void _doLogin()
	throws	FtpClientException,
			IOException
{
	cOutput.println("USER " + _sUser);
	cOutput.flush();
	_sAns = _readCSock();
	//System.out.println(_sAns);
	if(_sAns.charAt(0) == '2'){
	}
	else if(_sAns.charAt(0) == '3'){
		cOutput.println("PASS "+_sPass);
		cOutput.flush();
		_sAns = _readCSock();
		if(_sAns.charAt(0) == '2'){
		}
		else{
			throw new FtpClientException("bad password.");
		}
	}
	else{
		throw new FtpClientException("unknown response.");
	}
}
private String _doLs()
{
	return(_doLs(""));
}
private String _doLs(String sFile)
{
	String	sRc = "";
	try{
		int n;
		byte[] buff = new byte[1024];
		String sList = "LIST";
		if(! sFile.equals("")){
			sList=sList+" "+sFile;
		}
		Socket				dataSocket	= _createDataSock(sList);
		BufferedInputStream dataInput	= new BufferedInputStream(dataSocket.getInputStream());
		while((n = dataInput.read(buff)) > 0){
			sRc = sRc + new String(buff,0,n);
		}
		dataSocket.close();
		_readCSock();
	}
	catch(Exception e){
		e.printStackTrace();
	}
	return(sRc);
}
private String _doMkDir(String sDirectory)
{
	String	sRc = "";
	return(sRc);
}
private void _doPut(String sRemoteFile, String sLocalFile)
	throws	IOException
{
	int n;
	byte[] buff = new byte[1024];
	FileInputStream	inf		= new FileInputStream(sLocalFile);
	Socket			ds		= _createDataSock("STOR " + sRemoteFile);
	OutputStream	os		= ds.getOutputStream();
	while((n = inf.read(buff)) > 0){
		os.write(buff,0,n);
	}
	ds	.close();
	inf	.close();
	_readCSock();
}
private void _doQuit()
{
	try{
		cOutput.println("QUIT");
		cOutput.flush();
		_readCSock();
	}
	catch(Exception e){
		e.printStackTrace();
	}
}
private String _doRmDir(String sDirectory)
{
	String	sRc = "";
	return(sRc);
}
private String _readCSock()
	throws	UnknownHostException,
			IOException
{
	String sRc = "";
	while(! cInput.ready()){};
	while(cInput.ready()){
		sRc = sRc + cInput.readLine();
	}
	return(sRc);
}
public String ftpCmdCD(String sDirectory)
	throws	UnknownHostException,
			IOException
{
	if(sDirectory == null){
		throw new NullPointerException();
	}
	else if(sDirectory.equals("")){
		throw new NullPointerException();
	}
	else{
		return(_doFtpControl("CWD "+sDirectory));
	}
}
public String ftpCmdLS(String sDirectory)
	throws	UnknownHostException,
			IOException
{
	if(sDirectory == null){
		throw new NullPointerException();
	}
	else if(sDirectory.equals("")){
		return(_doFtpCommand("LIST"));
	}
	else{
		return(_doFtpCommand("LIST "+sDirectory));
	}
}
public String ftpCmdPWD()
	throws	UnknownHostException,
			IOException
{
	return(_doFtpCommand("PWD"));
}
public String ftpCmdSYST()
	throws	UnknownHostException,
			IOException
{
	return(_doFtpCommand("SYST"));
}
public void ftpGetAscii(String sRemoteFile, String sLocalFile)
	throws	IOException
{
	_doAscii	();
	_doGet	(sRemoteFile,sLocalFile);
}
public void ftpGetBinary(String sRemoteFile, String sLocalFile)
	throws	IOException
{
	_doBinary	();
	_doGet		(sRemoteFile,sLocalFile);
}
public void ftpLogin(String host, String user, String pass)
	throws	UnknownHostException,
			IOException,
			FtpClientException
{
	_sHost = host;
	_createSockets();
	_sUser = user;
	_sPass = pass;
	_doLogin();
}
public void ftpLogout()
	throws	UnknownHostException,
			IOException
{
	_doQuit			();
	_closeSockets	();
}
public void ftpPutAscii(String sRemoteFile, String sLocalFile)
	throws	IOException
{
	_doAscii	();
	_doPut		(sRemoteFile,sLocalFile);
}
public void ftpPutBinary(String sRemoteFile, String sLocalFile)
	throws	IOException
{
	_doBinary	();
	_doPut		(sRemoteFile,sLocalFile);
}
}
